// belief bias MPT
data {
  int<lower=1> nsubj;
  real prior_sd_diff;

  int<lower=0> vbd[nsubj,3];
  int<lower=0> ibd[nsubj,3];
  int<lower=0> vud[nsubj,3];
  int<lower=0> iud[nsubj,3];

}
transformed data {
  int<lower=1> nparams;

  // Observed frequencies: needed for T1 test statistic
  real n_kj_obs[4, 3];

  for (i in 1:4) {
    for (j in 1:3) {
      n_kj_obs[i, j] = 0;
    }
  }

  nparams = 8;

  for (t in 1:nsubj){
    for (i in 1:3) {
      n_kj_obs[1,i] = n_kj_obs[1,i] +  vbd[t,i];
      n_kj_obs[2,i] = n_kj_obs[2,i] +  ibd[t,i];
      n_kj_obs[3,i] = n_kj_obs[3,i] +  vud[t,i];
      n_kj_obs[4,i] = n_kj_obs[4,i] +  iud[t,i];
    }
  }
  for (i in 1:4) {
    for (j in 1:3) {
      n_kj_obs[i, j] =   n_kj_obs[i, j] / nsubj;
    }
  }
}

parameters {
  // hierarchical structure
  real mu_ri_u;
  real delta_ri;
  real mu_rv_u;
  real delta_rv;
  real mu_n_u;
  real delta_n;
  real mu_g_u;
  real delta_g;

  matrix[nparams,nsubj] deltahat_tilde;
  cholesky_factor_corr[nparams] L_Omega;
  vector<lower=0>[nparams] sigma;

}
transformed parameters {
  // predictions
  simplex[3] theta_ib[nsubj];
  simplex[3] theta_iu[nsubj];
  simplex[3] theta_vb[nsubj];
  simplex[3] theta_vu[nsubj];

  // transformed individual parameters
  real<lower=0,upper=1> rib[nsubj];
  real<lower=0,upper=1> rvb[nsubj];
  real<lower=0,upper=1> riu[nsubj];
  real<lower=0,upper=1> rvu[nsubj];
  real<lower=0,upper=1> nb[nsubj];
  real<lower=0,upper=1> nu[nsubj];
  real<lower=0,upper=1> gb[nsubj];
  real<lower=0,upper=1> gu[nsubj];

  matrix[nsubj,nparams] deltahat;
  deltahat = (diag_pre_multiply(sigma, L_Omega) * deltahat_tilde)';

  for (j in 1:nsubj) {

    // add individual displacement to group level parameters and transform if necessary:
    rib[j] = Phi(mu_ri_u + delta_ri/2 + deltahat[j,1]);
    riu[j] = Phi(mu_ri_u - delta_ri/2 + deltahat[j,2]);
    rvb[j] = Phi(mu_rv_u + delta_rv/2 + deltahat[j,3]);
    rvu[j] = Phi(mu_rv_u - delta_rv/2 + deltahat[j,4]);
    gb[j] = Phi(mu_g_u + delta_g/2 + deltahat[j,5]);
    gu[j] = Phi(mu_g_u - delta_g/2 + deltahat[j,6]);
    nb[j] = Phi(mu_n_u + delta_n/2 + deltahat[j,7]);
    nu[j] = Phi(mu_n_u - delta_n/2  + deltahat[j,8]);


    theta_vb[j, 1] = rvb[j] + (1-rvb[j])*(1-nb[j])*gb[j];
    theta_vb[j, 2] = (1-rvb[j])*nb[j];
    theta_vb[j, 3] = (1-rvb[j])*(1-nb[j])*(1-gb[j]);

    theta_ib[j, 1] = (1-rib[j])*(1-nb[j])*gb[j];
    theta_ib[j, 2] = (1-rib[j])*nb[j];
    theta_ib[j, 3] = rib[j] + (1-rib[j])*(1-nb[j])*(1-gb[j]);

    theta_vu[j, 1] = rvu[j] + (1-rvu[j])*(1-nu[j])*gu[j];
    theta_vu[j, 2] = (1-rvu[j])*nu[j];
    theta_vu[j, 3] = (1-rvu[j])*(1-nu[j])*(1-gu[j]);

    theta_iu[j, 1] = (1-riu[j])*(1-nu[j])*gu[j];
    theta_iu[j, 2] = (1-riu[j])*nu[j];
    theta_iu[j, 3] = riu[j] + (1-riu[j])*(1-nu[j])*(1-gu[j]);

  }

}
model {
  // prios of hyperparameters
  mu_ri_u ~ normal(0, 1);
  delta_ri ~ normal(0, prior_sd_diff);
  mu_rv_u ~ normal(0, 1);
  delta_rv ~ normal(0, prior_sd_diff);
  mu_n_u ~ normal(0, 1);
  delta_n ~ normal(0, prior_sd_diff);
  mu_g_u ~ normal(0, 1);
  delta_g ~ normal(0, prior_sd_diff);

  L_Omega ~ lkj_corr_cholesky(4);
  sigma ~ normal(0, 1);
  to_vector(deltahat_tilde) ~ normal(0, 1);

  // Observed counts
  for (t in 1:nsubj) {
    vbd[t,] ~ multinomial(theta_vb[t]);
    ibd[t,] ~ multinomial(theta_ib[t]);
    vud[t,] ~ multinomial(theta_vu[t]);
    iud[t,] ~ multinomial(theta_iu[t]);
  }
}
generated quantities {
  real mu_rib;
  real mu_rvb;
  real mu_riu;
  real mu_rvu;
  real mu_nb;
  real mu_nu;
  real mu_gb;
  real mu_gu;

  real T1_obs = 0;
  real T1_sim  = 0;
  real nh_kj[4, 3];
  real n_kj_sim[4, 3];
  int T1;
  corr_matrix[nparams] Omega;

  int<lower=0> vb_sim[nsubj,3];
  int<lower=0> ib_sim[nsubj,3];
  int<lower=0> vu_sim[nsubj,3];
  int<lower=0> iu_sim[nsubj,3];

  for (i in 1:4) {
    for (j in 1:3) {
      nh_kj[i, j] = 0;
    }
  }

  for (i in 1:4) {
    for (j in 1:3) {
      n_kj_sim[i, j] = 0;
    }
  }

  mu_rib = Phi(mu_ri_u + delta_ri/2);
  mu_riu = Phi(mu_ri_u - delta_ri/2);
  mu_rvb = Phi(mu_rv_u + delta_rv/2);
  mu_rvu = Phi(mu_rv_u - delta_rv/2);

  mu_nb = Phi(mu_n_u + delta_n/2);
  mu_nu = Phi(mu_n_u - delta_n/2);
  mu_gb = Phi(mu_g_u + delta_g/2);
  mu_gu = Phi(mu_g_u - delta_g/2);

  Omega = multiply_lower_tri_self_transpose(L_Omega);


  // Posterior predivtive test with test statistic T1
  for (t in 1:nsubj) {

    vb_sim[t,] = multinomial_rng(theta_vb[t], 2);
    ib_sim[t,] = multinomial_rng(theta_ib[t], 2);
    vu_sim[t,] = multinomial_rng(theta_vu[t], 2);
    iu_sim[t,] = multinomial_rng(theta_iu[t], 2);

    for (i in 1:3) {
     nh_kj[1,i] = nh_kj[1,i] + theta_vb[t,i]*2;
     nh_kj[2,i] = nh_kj[2,i] + theta_ib[t,i]*2;
     nh_kj[3,i] = nh_kj[3,i] + theta_vu[t,i]*2;
     nh_kj[4,i] = nh_kj[4,i] + theta_iu[t,i]*2;

     n_kj_sim[1,i] = n_kj_sim[1,i] + vb_sim[t,i];
     n_kj_sim[2,i] = n_kj_sim[2,i] + ib_sim[t,i];
     n_kj_sim[3,i] = n_kj_sim[3,i] + vu_sim[t,i];
     n_kj_sim[4,i] = n_kj_sim[4,i] + iu_sim[t,i];
    }
  }

  for (i in 1:4) {
   for (j in 1:3) {
    nh_kj[i,j] = nh_kj[i,j]  / nsubj;
    n_kj_sim[i,j] = n_kj_sim[i,j] / nsubj;
    }
  }

 // T1_obs and T1_sim //

 for (i in 1:4) {
  for (j in 1:3) {
   T1_obs += ((n_kj_obs[i,j] - nh_kj[i,j])^2 / nh_kj[i,j] );
   T1_sim += ((n_kj_sim[i,j] - nh_kj[i,j])^2 / nh_kj[i,j] );
   }
 }

 // T1 //
 T1 = T1_sim > T1_obs;

}
